from .uptimerobot import UpTimeRobot
from .keep_alive import keep_alive
from .vars import slug, url

__all__ = (
    'lazy_setup',
)

def lazy_setup(uptimerobot_api_key: str):
    """Sets everything up"""
    prefix = '\033[96mlazy setup:'
    postfix = '\033[0m'
    api = UpTimeRobot(uptimerobot_api_key)
    print(prefix, 'Starting webserver...',postfix)
    keep_alive()
    print(prefix, 'Setting up UpTimeRobot monitor for this slug',postfix)
    api.sync_upsert_monitor(friendly_name=f'Bot:{slug}', url=url)
    print(prefix, 'Lazy Setup complete.', postfix)



import aiohttp
from aiolimiter import AsyncLimiter
import asyncio
from .monitor import Monitor

__all__ = (
    'UpTimeRobot',
)

api_version = 'v2'

base_url = f'https://api.uptimerobot.com/{api_version}/'

class UpTimeRobot:
    api_key: str
    max_rate: int

    def __init__(self, api_key, max_rate = 10): 
        self.key = api_key
        self.base_payload = f'api_key={api_key}&format=json'
        self.limiter = AsyncLimiter(max_rate=max_rate)
        self.headers = headers = {
            'content-type': "application/x-www-form-urlencoded",
            'cache-control': "no-cache"
        }
    
    async def get_monitors(self):
        """Returns the monitors set for this account"""
        payload = base_url + 'getMonitors?' + self.base_payload
        async with self.limiter:
            async with aiohttp.ClientSession(headers=self.headers) as cs:
                async with cs.post(payload) as r:
                    data = await r.json()
        if r.status == 200:
            return [Monitor(m) for m in data.get('monitors',[])]
        else:
            raise Exception(f'Something went wrong: {data}')
    
    async def new_monitor(self, friendly_name: str, url: str):
        """Configures a new monitor"""
        post_url = base_url+'newMonitor'
        payload = {
            'api_key': self.key,
            'format': 'json',
            'type': '1',
            'friendly_name': friendly_name,
            'url': url
        }
        async with self.limiter:
            async with aiohttp.ClientSession(headers=self.headers) as cs:
                async with cs.post(post_url, data=payload) as r:
                    data = await r.json()
        if data['stat'] != 'ok':
            raise Exception(f'Something went wrong |[{r.status}] data:{data}')
    
    async def upsert_monitor(self, friendly_name: str, url: str):
        monitors = await self.get_monitors()
        if not any(
            monitor for monitor in monitors
            if monitor.name == friendly_name or monitor.url == url
        ):
            await self.new_monitor(friendly_name, url)
    
    def sync_upsert_monitor(self, friendly_name: str, url: str):
        asyncio.get_event_loop().run_until_complete(self.upsert_monitor(friendly_name, url))
    
    def sync_get_monitors(self):
        return asyncio.get_event_loop().run_until_complete(self.get_monitors())




__all__ = (
    'Monitor',
)

class Monitor:
    def __init__(self, data):
        self.id = data.get('id')
        self.name = data.get('friendly_name')
        self.url = data.get('url')
    
    def __repr__(self) -> str:
        return f'Monitor(name={self.name}, url={self.url})'

    def __str__(self) -> str:
        return 'monitor: ' + self.name






# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dontuserepl', 'dontuserepl.uptimerobot']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp==3.7.4.post0', 'aiolimiter==1.0.0b1']

setup_kwargs = {
    'name': 'dontuserepl',
    'version': '0.1.3',
    'description': 'Simple tools to deploy a script 24/7 on repl.it using an aiohttp server and automated uptimerobot monitors',
    'long_description': '# dontuserepl\n\nLet\'s face it, repl.it is one of the best places to develop, test and run code. As such, it\'s been popular for small discord bots for a couple of years now. Many people dislike the platform for various reasons, mainly, low resources, public code, etc, and they\'ll tell you to stop using it, however it remains a good place to start out and try new code easily.\n\nThe most popular way to run a discord bot on repl is to create a webserver on a different thread and configure a monitor service like uptimerobot.com to ping the server every five minutes or so.\n\nEven though this is very simple to do, i feel bored by it, that\'s why i wrote this simple library.\n\n# How to use it:\nMaking your bot 24/7 is extremely easy with `dontuserepl`.\n\n1) Go to https://uptimerobot.com/, open an account and login.\n2) Go to "My settings" and scroll to "API Settings".\n3) Create and copy a "Main API Key"\n4) Go to your repl and add the key as a secret\n5) Add the snippet below to your main.py file.\n```python\nimport os\nfrom dontuserepl import lazy_setup\nkey = os.getenv(\'uptimerobot_api_key\')  # use the name of the secret from step 4\nlazy_setup(key)\n```\n6) That\'s it, `lazy_setup` runs a minimal aiohttp server on port 8080 and configures a monitor for the script.\n\n\n# Working example\n```python\nimport os\nfrom dontuserepl import lazy_setup\nfrom discord.ext import commands\n\nkey = os.getenv(\'uptimerobot_api_key\')\ntoken = os.getenv(\'discord_token\')\n\nbot = commands.Bot(command_prefix=\'!\')\n\n@bot.event\nasync def on_ready():\n    print(f\'logged in as "{bot.user.name}"\')\n\n@bot.command()\nasync def ping(ctx):\n    await ctx.send(\'pong\')\n\n\nlazy_setup(key) # don\'t forget to call this *before* running the bot\nbot.run(token)\n```\n\n# todo:\n- [X] Upload to pypi\n- [ ] Documentation\n- [ ] Better error handling\n',
    'author': 'chrisdewa',
    'author_email': 'alexdewa@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/chrisdewa/dontuserepl',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import asyncio

from aiohttp import web

__all__ = (
    'keep_alive',
)

def web_server() -> web.AppRunner:
    def main(request):
        return web.Response(text="I'm alive")
        
    app = web.Application()
    app.add_routes([web.get('/', main)])
    runner = web.AppRunner(app)
    return runner

def keep_alive():
    runner = web_server()
    loop = asyncio.get_event_loop()
    loop.run_until_complete(runner.setup())
    site = web.TCPSite(runner, '0.0.0.0', 8080)
    loop.run_until_complete(site.start())
